@@ -18,22 +18,22 @@
 
 from __future__ import annotations
 
-from datafusion._internal import functions as f
-from datafusion.expr import (
-    CaseBuilder,
-    Expr,
-    WindowFrame,
-    SortExpr,
-    sort_list_to_raw_sort_list,
-    expr_list_to_raw_expr_list,
-)
-from datafusion.context import SessionContext
-from datafusion.common import NullTreatment
-
 from typing import Any, Optional
 
 import pyarrow as pa
 
+from datafusion._internal import functions as f
+from datafusion.common import NullTreatment
+from datafusion.context import SessionContext
+from datafusion.expr import (
+    CaseBuilder,
+    Expr,
+    SortExpr,
+    WindowFrame,
+    expr_list_to_raw_expr_list,
+    sort_list_to_raw_sort_list,
+)
+
 __all__ = [
     "abs",
     "acos",
@@ -82,6 +82,7 @@ __all__ = [
     "array_to_string",
     "array_union",
     "arrow_typeof",
+    "arrow_cast",
     "ascii",
     "asin",
     "asinh",
@@ -1109,6 +1110,11 @@ def arrow_typeof(arg: Expr) -> Expr:
     return Expr(f.arrow_typeof(arg.expr))
 
 
+def arrow_cast(expr: Expr, data_type: Expr) -> Expr:
+    """Casts an expression to a specified data type."""
+    return Expr(f.arrow_cast(expr.expr, data_type.expr))
+
+
 def random() -> Expr:
     """Returns a random value in the range ``0.0 <= x < 1.0``."""
     return Expr(f.random())
