@@ -26,7 +26,10 @@ from __future__ import annotations
 import pathlib
 from typing import TYPE_CHECKING
 
-from typing_extensions import deprecated
+try:
+    from warnings import deprecated  # Python 3.13+
+except ImportError:
+    from typing_extensions import deprecated  # Python 3.12
 
 from datafusion.plan import LogicalPlan
 
