@@ -22,8 +22,10 @@ use datafusion::logical_expr::WindowFrame;
 use pyo3::{prelude::*, wrap_pyfunction};
 
 use crate::common::data_type::NullTreatment;
+use crate::common::data_type::PyScalarValue;
 use crate::context::PySessionContext;
-use crate::errors::DataFusionError;
+use crate::errors::PyDataFusionError;
+use crate::errors::PyDataFusionResult;
 use crate::expr::conditional_expr::PyCaseBuilder;
 use crate::expr::sort_expr::to_sort_expressions;
 use crate::expr::sort_expr::PySortExpr;
@@ -36,10 +38,7 @@ use datafusion::functions_aggregate;
 use datafusion::functions_window;
 use datafusion::logical_expr::expr::Alias;
 use datafusion::logical_expr::sqlparser::ast::NullTreatment as DFNullTreatment;
-use datafusion::logical_expr::{
-    expr::{find_df_window_func, WindowFunction},
-    lit, Expr, WindowFunctionDefinition,
-};
+use datafusion::logical_expr::{expr::WindowFunction, lit, Expr, WindowFunctionDefinition};
 
 fn add_builder_fns_to_aggregate(
     agg_fn: Expr,
@@ -47,7 +46,7 @@ fn add_builder_fns_to_aggregate(
     filter: Option<PyExpr>,
     order_by: Option<Vec<PySortExpr>>,
     null_treatment: Option<NullTreatment>,
-) -> PyResult<PyExpr> {
+) -> PyDataFusionResult<PyExpr> {
     // Since ExprFuncBuilder::new() is private, we can guarantee initializing
     // a builder with an `null_treatment` with option None
     let mut builder = agg_fn.null_treatment(None);
@@ -231,13 +230,10 @@ fn when(when: PyExpr, then: PyExpr) -> PyResult<PyCaseBuilder> {
 /// 1) If no function has been found, search default aggregate functions.
 ///
 /// NOTE: we search the built-ins first because the `UDAF` versions currently do not have the same behavior.
-fn find_window_fn(name: &str, ctx: Option<PySessionContext>) -> PyResult<WindowFunctionDefinition> {
-    // search built in window functions (soon to be deprecated)
-    let df_window_func = find_df_window_func(name);
-    if let Some(df_window_func) = df_window_func {
-        return Ok(df_window_func);
-    }
-
+fn find_window_fn(
+    name: &str,
+    ctx: Option<PySessionContext>,
+) -> PyDataFusionResult<WindowFunctionDefinition> {
     if let Some(ctx) = ctx {
         // search UDAFs
         let udaf = ctx
@@ -293,7 +289,9 @@ fn find_window_fn(name: &str, ctx: Option<PySessionContext>) -> PyResult<WindowF
         return Ok(window_fn);
     }
 
-    Err(DataFusionError::Common(format!("window function `{name}` not found")).into())
+    Err(PyDataFusionError::Common(format!(
+        "window function `{name}` not found"
+    )))
 }
 
 /// Creates a new Window function expression
@@ -350,7 +348,7 @@ macro_rules! aggregate_function {
             filter: Option<PyExpr>,
             order_by: Option<Vec<PySortExpr>>,
             null_treatment: Option<NullTreatment>
-        ) -> PyResult<PyExpr> {
+        ) -> PyDataFusionResult<PyExpr> {
             let agg_fn = functions_aggregate::expr_fn::$NAME($($arg.into()),*);
 
             add_builder_fns_to_aggregate(agg_fn, distinct, filter, order_by, null_treatment)
@@ -371,7 +369,7 @@ macro_rules! aggregate_function_vec_args {
             filter: Option<PyExpr>,
             order_by: Option<Vec<PySortExpr>>,
             null_treatment: Option<NullTreatment>
-        ) -> PyResult<PyExpr> {
+        ) -> PyDataFusionResult<PyExpr> {
             let agg_fn = functions_aggregate::expr_fn::$NAME(vec![$($arg.into()),*]);
 
             add_builder_fns_to_aggregate(agg_fn, distinct, filter, order_by, null_treatment)
@@ -400,7 +398,6 @@ macro_rules! expr_fn {
         }
     };
 }
-
 /// Generates a [pyo3] wrapper for [datafusion::functions::expr_fn]
 ///
 /// These functions take a single `Vec<PyExpr>` argument using `pyo3(signature = (*args))`.
@@ -575,6 +572,7 @@ expr_fn_vec!(r#struct); // Use raw identifier since struct is a keyword
 expr_fn_vec!(named_struct);
 expr_fn!(from_unixtime, unixtime);
 expr_fn!(arrow_typeof, arg_1);
+expr_fn!(arrow_cast, arg_1 datatype);
 expr_fn!(random);
 
 // Array Functions
@@ -651,7 +649,7 @@ pub fn approx_percentile_cont(
     percentile: f64,
     num_centroids: Option<i64>, // enforces optional arguments at the end, currently
     filter: Option<PyExpr>,
-) -> PyResult<PyExpr> {
+) -> PyDataFusionResult<PyExpr> {
     let args = if let Some(num_centroids) = num_centroids {
         vec![expression.expr, lit(percentile), lit(num_centroids)]
     } else {
@@ -670,7 +668,7 @@ pub fn approx_percentile_cont_with_weight(
     weight: PyExpr,
     percentile: f64,
     filter: Option<PyExpr>,
-) -> PyResult<PyExpr> {
+) -> PyDataFusionResult<PyExpr> {
     let agg_fn = functions_aggregate::expr_fn::approx_percentile_cont_with_weight(
         expression.expr,
         weight.expr,
@@ -692,7 +690,7 @@ pub fn first_value(
     filter: Option<PyExpr>,
     order_by: Option<Vec<PySortExpr>>,
     null_treatment: Option<NullTreatment>,
-) -> PyResult<PyExpr> {
+) -> PyDataFusionResult<PyExpr> {
     // If we initialize the UDAF with order_by directly, then it gets over-written by the builder
     let agg_fn = functions_aggregate::expr_fn::first_value(expr.expr, None);
 
@@ -709,7 +707,7 @@ pub fn nth_value(
     filter: Option<PyExpr>,
     order_by: Option<Vec<PySortExpr>>,
     null_treatment: Option<NullTreatment>,
-) -> PyResult<PyExpr> {
+) -> PyDataFusionResult<PyExpr> {
     let agg_fn = datafusion::functions_aggregate::nth_value::nth_value(expr.expr, n, vec![]);
     add_builder_fns_to_aggregate(agg_fn, distinct, filter, order_by, null_treatment)
 }
@@ -724,7 +722,7 @@ pub fn string_agg(
     filter: Option<PyExpr>,
     order_by: Option<Vec<PySortExpr>>,
     null_treatment: Option<NullTreatment>,
-) -> PyResult<PyExpr> {
+) -> PyDataFusionResult<PyExpr> {
     let agg_fn = datafusion::functions_aggregate::string_agg::string_agg(expr.expr, lit(delimiter));
     add_builder_fns_to_aggregate(agg_fn, distinct, filter, order_by, null_treatment)
 }
@@ -735,7 +733,7 @@ pub(crate) fn add_builder_fns_to_window(
     window_frame: Option<PyWindowFrame>,
     order_by: Option<Vec<PySortExpr>>,
     null_treatment: Option<NullTreatment>,
-) -> PyResult<PyExpr> {
+) -> PyDataFusionResult<PyExpr> {
     let null_treatment = null_treatment.map(|n| n.into());
     let mut builder = window_fn.null_treatment(null_treatment);
 
@@ -757,7 +755,7 @@ pub(crate) fn add_builder_fns_to_window(
         builder = builder.window_frame(window_frame.into());
     }
 
-    builder.build().map(|e| e.into()).map_err(|err| err.into())
+    Ok(builder.build().map(|e| e.into())?)
 }
 
 #[pyfunction]
@@ -765,10 +763,11 @@ pub(crate) fn add_builder_fns_to_window(
 pub fn lead(
     arg: PyExpr,
     shift_offset: i64,
-    default_value: Option<ScalarValue>,
+    default_value: Option<PyScalarValue>,
     partition_by: Option<Vec<PyExpr>>,
     order_by: Option<Vec<PySortExpr>>,
-) -> PyResult<PyExpr> {
+) -> PyDataFusionResult<PyExpr> {
+    let default_value = default_value.map(|v| v.into());
     let window_fn = functions_window::expr_fn::lead(arg.expr, Some(shift_offset), default_value);
 
     add_builder_fns_to_window(window_fn, partition_by, None, order_by, None)
@@ -779,10 +778,11 @@ pub fn lead(
 pub fn lag(
     arg: PyExpr,
     shift_offset: i64,
-    default_value: Option<ScalarValue>,
+    default_value: Option<PyScalarValue>,
     partition_by: Option<Vec<PyExpr>>,
     order_by: Option<Vec<PySortExpr>>,
-) -> PyResult<PyExpr> {
+) -> PyDataFusionResult<PyExpr> {
+    let default_value = default_value.map(|v| v.into());
     let window_fn = functions_window::expr_fn::lag(arg.expr, Some(shift_offset), default_value);
 
     add_builder_fns_to_window(window_fn, partition_by, None, order_by, None)
@@ -793,7 +793,7 @@ pub fn lag(
 pub fn row_number(
     partition_by: Option<Vec<PyExpr>>,
     order_by: Option<Vec<PySortExpr>>,
-) -> PyResult<PyExpr> {
+) -> PyDataFusionResult<PyExpr> {
     let window_fn = functions_window::expr_fn::row_number();
 
     add_builder_fns_to_window(window_fn, partition_by, None, order_by, None)
@@ -804,7 +804,7 @@ pub fn row_number(
 pub fn rank(
     partition_by: Option<Vec<PyExpr>>,
     order_by: Option<Vec<PySortExpr>>,
-) -> PyResult<PyExpr> {
+) -> PyDataFusionResult<PyExpr> {
     let window_fn = functions_window::expr_fn::rank();
 
     add_builder_fns_to_window(window_fn, partition_by, None, order_by, None)
@@ -815,7 +815,7 @@ pub fn rank(
 pub fn dense_rank(
     partition_by: Option<Vec<PyExpr>>,
     order_by: Option<Vec<PySortExpr>>,
-) -> PyResult<PyExpr> {
+) -> PyDataFusionResult<PyExpr> {
     let window_fn = functions_window::expr_fn::dense_rank();
 
     add_builder_fns_to_window(window_fn, partition_by, None, order_by, None)
@@ -826,7 +826,7 @@ pub fn dense_rank(
 pub fn percent_rank(
     partition_by: Option<Vec<PyExpr>>,
     order_by: Option<Vec<PySortExpr>>,
-) -> PyResult<PyExpr> {
+) -> PyDataFusionResult<PyExpr> {
     let window_fn = functions_window::expr_fn::percent_rank();
 
     add_builder_fns_to_window(window_fn, partition_by, None, order_by, None)
@@ -837,7 +837,7 @@ pub fn percent_rank(
 pub fn cume_dist(
     partition_by: Option<Vec<PyExpr>>,
     order_by: Option<Vec<PySortExpr>>,
-) -> PyResult<PyExpr> {
+) -> PyDataFusionResult<PyExpr> {
     let window_fn = functions_window::expr_fn::cume_dist();
 
     add_builder_fns_to_window(window_fn, partition_by, None, order_by, None)
@@ -849,7 +849,7 @@ pub fn ntile(
     arg: PyExpr,
     partition_by: Option<Vec<PyExpr>>,
     order_by: Option<Vec<PySortExpr>>,
-) -> PyResult<PyExpr> {
+) -> PyDataFusionResult<PyExpr> {
     let window_fn = functions_window::expr_fn::ntile(arg.into());
 
     add_builder_fns_to_window(window_fn, partition_by, None, order_by, None)
@@ -867,6 +867,7 @@ pub(crate) fn init_module(m: &Bound<'_, PyModule>) -> PyResult<()> {
     m.add_wrapped(wrap_pyfunction!(range))?;
     m.add_wrapped(wrap_pyfunction!(array_agg))?;
     m.add_wrapped(wrap_pyfunction!(arrow_typeof))?;
+    m.add_wrapped(wrap_pyfunction!(arrow_cast))?;
     m.add_wrapped(wrap_pyfunction!(ascii))?;
     m.add_wrapped(wrap_pyfunction!(asin))?;
     m.add_wrapped(wrap_pyfunction!(asinh))?;
