@@ -22,12 +22,11 @@ use std::sync::Arc;
 use arrow::array::{make_array, Array, ArrayData, ArrayRef};
 use datafusion::logical_expr::function::{PartitionEvaluatorArgs, WindowUDFFieldArgs};
 use datafusion::logical_expr::window_state::WindowAggState;
-use datafusion::physical_plan::PhysicalExpr;
 use datafusion::scalar::ScalarValue;
-use datafusion_functions_window_common::expr::ExpressionArgs;
 use pyo3::exceptions::PyValueError;
 use pyo3::prelude::*;
 
+use crate::common::data_type::PyScalarValue;
 use crate::expr::PyExpr;
 use crate::utils::parse_volatility;
 use datafusion::arrow::datatypes::DataType;
@@ -135,7 +134,8 @@ impl PartitionEvaluator for RustPartitionEvaluator {
             self.evaluator
                 .bind(py)
                 .call_method1("evaluate", py_args)
-                .and_then(|v| v.extract())
+                .and_then(|v| v.extract::<PyScalarValue>())
+                .map(|v| v.0)
                 .map_err(|e| DataFusionError::Execution(format!("{e}")))
         })
     }
@@ -319,8 +319,4 @@ impl WindowUDFImpl for MultiColumnWindowUDF {
         let _ = _partition_evaluator_args;
         (self.partition_evaluator_factory)()
     }
-
-    fn expressions(&self, expr_args: ExpressionArgs) -> Vec<Arc<dyn PhysicalExpr>> {
-        expr_args.input_exprs().into()
-    }
 }
